@@ -1,17 +1,12 @@
 import collections
+from copy import deepcopy
 from enum import Enum
 import os
-import sys
 
 import requests
 
-up = os.path.dirname
-# <submodule>/api/workflow/common.py
-submodule_directory = up(up(up(__file__)))
-sys.path.append(submodule_directory)
-
-from api.base import GenericAPI, ERROR_THRESHOLD
-import utils
+from .base import GenericAPI, ERROR_THRESHOLD
+from .. import utils
 
 
 class FilesAPI(GenericAPI):
@@ -337,6 +332,146 @@ class FilesAPI(GenericAPI):
         folder_id = r.json()['document']['content']['pk']
         return folder_id
 
+    def move(self,
+             from_space=None, from_space_id=None, from_paths=None,
+             file_ids=None,
+             to_space=None, to_space_id=None, to_directory=None,
+             to_directory_id=None,
+             ):
+        """
+        Move files from one file system location to another.
+
+        Parameters
+        ----------
+        from_space : str or None
+            Will only be used if file_ids is None.
+            Type of space where files currently exist.
+            Must be 'group', 'project', or 'user'.
+        from_space_id : str or None
+            Will only be used if file_ids is None.
+            ID of space where files currently exist.
+            Group or project id, or username.
+        from_paths : (list of str) or None
+            Will only be used if file_ids is None.
+            List of human-readable paths where files currently exist.
+        file_ids : (list of str) or None
+            IDs of files to be moved.
+            If None, from_space, from_space_id, and from_paths must be supplied.
+        to_space : str or None
+            Will only be used if to_directory_id is None.
+            Type of space where files should be moved.
+            Must be 'group', 'project', or 'user'.
+        to_space_id : str or None
+            Will only be used if to_directory_id is None.
+            ID of space where files should be moved.
+            Group or project id, or username.
+        to_directory : str or None
+            Will only be used if to_directory_id is None.
+            Human-readable path to destination directory in destination space.
+        to_directory_id : str or None
+            ID of directory where files should be moved.
+            If None, to_space, to_space_id, and to_directory must be supplied.
+            Cannot be used if files are moved to the root directory of the
+            destination space, since the root directory has no id.
+
+        Returns
+        -------
+        Data on the enqueueing operation, including the number of items
+        processed and queue status for each item.
+        """
+        if (
+            file_ids is None
+            and
+            (
+                from_space is None
+                or
+                from_space_id is None
+                or
+                from_paths is None
+            )
+        ):
+            raise ValueError(
+                "Either the from_ids parameter or all three of the "
+                "from_space, from_space_id, and from_paths parameters "
+                "must be supplied."
+            )
+
+        if (
+            to_directory_id is None
+            and
+            (
+                to_space is None
+                or
+                to_space_id is None
+                or
+                to_directory is None
+            )
+        ):
+            raise ValueError(
+                "Either the to_directory_id parameter or all three of the "
+                "to_space, to_space_id, and to_directory parameters "
+                "must be supplied."
+            )
+
+        if from_space is not None and from_space not in utils.VALID_SPACES:
+            raise ValueError(
+                "from_space must be None of one of "
+                f"{','.join(utils.VALID_SPACES)}")
+
+        if to_space is not None and to_space not in utils.VALID_SPACES:
+            raise ValueError(
+                "to_space must be None of one of "
+                f"{','.join(utils.VALID_SPACES)}")
+
+        if file_ids is not None and not isinstance(file_ids, list):
+            raise ValueError("file_ids must be None or a list")
+
+        if from_paths is not None and not isinstance(from_paths, list):
+            raise ValueError("from_paths must be None or a list")
+
+        common_data = {
+            'type': 'move',
+        }
+
+        # Use directory id if present.
+        if to_directory_id is not None:
+            common_data['toUuid'] = to_directory_id
+        else:
+            common_data['toSpaceType'] = to_space
+            common_data['toSpace'] = to_space_id
+            common_data['toDir'] = to_directory
+
+        data = []
+
+        # Use file_ids if present.
+        if file_ids is not None:
+            for file_id in file_ids:
+                item = deepcopy(common_data)
+                item['fromUuid'] = file_id
+                data.append(item)
+        else:
+            for from_path in from_paths:
+                item = deepcopy(common_data)
+                item['fromSpaceType'] = from_space
+                item['fromSpace'] = from_space_id
+                item['fromPath'] = from_path
+                data.append(item)
+
+        url = '{}/api/files/queue/'.format(self._auth.get_base_url())
+        headers = self._auth.get_headers()
+        cookies = self._auth.get_cookies()
+        r = requests.post(url, headers=headers, cookies=cookies,
+                          json=data, verify=False)
+
+        if r.status_code >= ERROR_THRESHOLD:
+            self._report_api_error(response=r)
+            return {
+                'processed': 0,
+                'results': 'ERROR'
+            }
+        else:
+            return r.json()
+
     def upload(self, local_path, space=None, space_id=None, path=None,
             metadata=None):
         """
@@ -640,9 +775,7 @@ class FilesAPI(GenericAPI):
                     'Content-Type': 'application/octet-stream'
                 })
 
-            print("kwargs", kwargs)
             r = requests.put(**kwargs)
-            print("r.request.headers", r.request.headers)
 
         if r.status_code >= ERROR_THRESHOLD:
             self._report_api_error(response=r)
